!--------------------------------------------------------------------------------------------------
!  DOXYGEN
!> @brief 1D SHAPIRO FILTER
!> @details 1D SHAPIRO FILTER.
!> @author ORIGINATOR - ?????
!> @date ??-??-?? \n
!> @author LUCCI 
!> @date 18-03-20 \n
!> @version V1.1.0
!> @details MODERNIZATION OF THE CODE, INCLUDING:
!!                      * F77 TO F90/F95
!!                      * INDENTATION & UNIFORMIZATION CODE
!!                      * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!!                      * DOCUMENTATION WITH DOXYGEN
!!                      * OPENMP FUNCTIONALITY 
!<
!> @param[in]    IMAX  - Significado de IMAX
!> @param[in]    ORDER - Significado de ORDER
!> @param[inout] HH1D  - Significado de HH1D 
!> @details <b>Use Module:</b>
!! @arg @c F77KINDS
!> @details <b>Driver:</b> 
!<
!> @details <b>Calls:</b>
!--------------------------------------------------------------------------------------------------     
	SUBROUTINE SHAP_1D(HH1D, IMAX, ORDER)
!--------------------------------------------------------------------------------------------------
! SUBROUTINE SHAP_1D
! 
! SUBPROGRAM: SHAP_1D - 1D SHAPIRO FILTER 
! PROGRAMMER: ????? 
! ORG: ?????
! DATE: ??-??-??
!
! ABSTRACT:  
! 1D SHAPIRO FILTER
!
! PROGRAM HISTORY LOG:
! ??-??-??  ?????   - ORIGINATOR
! 18-03-20  LUCCI   - MODERNIZATION OF THE CODE, INCLUDING:
!                     * F77 TO F90/F95
!                     * INDENTATION & UNIFORMIZATION CODE
!                     * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!                     * DOCUMENTATION WITH DOXYGEN
!                     * OPENMP FUNCTIONALITY 
!
!
! INPUT ARGUMENT LIST:
! IMAX  -
! ORDER - 
!
! OUTPUT ARGUMENT LIST:
! NONE
!
! INPUT/OUTPUT ARGUMENT LIST:
! HH1D  - 
!
! INPUT FILES:
! NONE
!
! OUTPUT FILES:
! NONE
!
! USE MODULES: F77KINDS
!
! DRIVER     : -----
!
! CALLS      : -----
!--------------------------------------------------------------------------------------------------
    USE F77KINDS
!
    IMPLICIT NONE
!
    INTEGER(KIND=I4)                                                      , INTENT(IN)          ::&
    & IMAX    , ORDER
!
    REAL   (KIND=R4)    , DIMENSION(-5:IMAX+6)                            , INTENT(INOUT)       ::&
    & HH1D
!
    REAL   (KIND=R4)    , DIMENSION(-5:IMAX+6)                                                  ::&
    & HHTMP1  , HHTMP2
!
   INTEGER(KIND=I4)                                                                             ::&
   & N        , I
!
    HHTMP1 = HH1D
!
    DO N=1,ORDER
        DO I=-4,IMAX+5
	    HHTMP2(I) = -0.25 * (HHTMP1(I+1) - 2 * HHTMP1(I) + HHTMP1(I-1))
        END DO
!
        HHTMP1 = HHTMP2
!
    END DO
!
    HH1D = HH1D - HHTMP1
!
    END SUBROUTINE SHAP_1D
!
!
!--------------------------------------------------------------------------------------------------
!  DOXYGEN
!> @brief 2D SHAPIRO FILTER
!> @details 2D SHAPIRO FILTER.
!> @author ORIGINATOR - ?????
!> @date ??-??-?? \n
!> @author LUCCI 
!> @date 18-03-20 \n
!> @version V1.1.0
!> @details MODERNIZATION OF THE CODE, INCLUDING:
!!                      * F77 TO F90/F95
!!                      * INDENTATION & UNIFORMIZATION CODE
!!                      * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!!                      * DOCUMENTATION WITH DOXYGEN
!!                      * OPENMP FUNCTIONALITY 
!<
!> @param[in]    NORD  - Significado de NORD
!> @param[inout] HH2D  - Significado de HH2D 
!> @details <b>Use Module:</b>
!! @arg @c F77KINDS
!! @arg @c PARMETA
!> @details <b>Driver:</b> 
!<
!> @details <b>Calls:</b>
!--------------------------------------------------------------------------------------------------     
    SUBROUTINE SHAP_2D(HH2D,NORD)
!--------------------------------------------------------------------------------------------------
! SUBROUTINE SHAP_2D
! 
! SUBPROGRAM: SHAP_2D - 2D SHAPIRO FILTER 
! PROGRAMMER: ????? 
! ORG: ?????
! DATE: ??-??-??
!
! ABSTRACT:  
! 2D SHAPIRO FILTER
!
! PROGRAM HISTORY LOG:
! ??-??-??  ?????   - ORIGINATOR
! 18-03-20  LUCCI   - MODERNIZATION OF THE CODE, INCLUDING:
!                     * F77 TO F90/F95
!                     * INDENTATION & UNIFORMIZATION CODE
!                     * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!                     * DOCUMENTATION WITH DOXYGEN
!                     * OPENMP FUNCTIONALITY 
!
!
! INPUT ARGUMENT LIST:
! NORD  - 
!
! OUTPUT ARGUMENT LIST:
! NONE
!
! INPUT/OUTPUT ARGUMENT LIST:
! HH2D  - 
!
! INPUT FILES:
! NONE
!
! OUTPUT FILES:
! NONE
!
! USE MODULES: F77KINDS
!              PARMETA
!
! DRIVER     : -----
!
! CALLS      : -----
!--------------------------------------------------------------------------------------------------
    USE F77KINDS
    USE PARMETA
!
    IMPLICIT NONE
!
    INTEGER(KIND=I4)                                                      , INTENT(IN)          ::&
    & NORD
!
    REAL   (KIND=R4)    ,DIMENSION(-5:IM+6,-5:JM+6)                       , INTENT(INOUT)       ::&
    & HH2D
! 
    INTEGER(KIND=I4)                                                                            ::&
    & N       , I       , J
!
    DO I=1,IM
	CALL SHAP_1D(HH2D(I,:), JM, NORD)
     END DO
!
    DO J=1,JM
        CALL SHAP_1D(HH2D(:,J), IM, NORD)
    END DO
!
    END SUBROUTINE SHAP_2D
!
!
!--------------------------------------------------------------------------------------------------
!  DOXYGEN
!> @brief Inserir Brief de SHAPIRO
!> @details Inserir Details de SHAPIRO
!> @author ORIGINATOR - ?????
!> @date ??-??-?? \n
!> @author LUCCI 
!> @date 18-03-20 \n
!> @version V1.1.0
!> @details MODERNIZATION OF THE CODE, INCLUDING:
!!                      * F77 TO F90/F95
!!                      * INDENTATION & UNIFORMIZATION CODE
!!                      * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!!                      * DOCUMENTATION WITH DOXYGEN
!!                      * OPENMP FUNCTIONALITY 
!<
!> @details <b>Use Module:</b>
!! @arg @c F77KINDS
!! @arg @c MPPSTAFF
!! @arg @c PARMETA
!! @arg @c VRBLS
!> @details <b>Driver:</b> 
!<
!> @details <b>Calls:</b>
!--------------------------------------------------------------------------------------------------  
    SUBROUTINE SHAPIRO
!--------------------------------------------------------------------------------------------------
! SUBROUTINE SHAPIRO
! 
! SUBPROGRAM: SHAPIRO - ?????
! PROGRAMMER: ????? 
! ORG: ?????
! DATE: ??-??-??
!
! ABSTRACT:  
! ?????
!
! PROGRAM HISTORY LOG:
! ??-??-??  ?????   - ORIGINATOR
! 18-03-20  LUCCI   - MODERNIZATION OF THE CODE, INCLUDING:
!                     * F77 TO F90/F95
!                     * INDENTATION & UNIFORMIZATION CODE
!                     * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!                     * DOCUMENTATION WITH DOXYGEN
!                     * OPENMP FUNCTIONALITY 
!
!
! INPUT ARGUMENT LIST:
! NORD  - 
!
! OUTPUT ARGUMENT LIST:
! NONE
!
! INPUT/OUTPUT ARGUMENT LIST:
! NONE
!
! INPUT FILES:
! NONE
!
! OUTPUT FILES:
! NONE
!
! USE MODULES: F77KINDS
!              MPPSTAFF
!              PARMETA
!              VRBLS
!
! DRIVER     : -----
!
! CALLS      : -----
!--------------------------------------------------------------------------------------------------
    USE F77KINDS
    USE MPPSTAFF
    USE PARMETA
    USE VRBLS
!
    IMPLICIT NONE
!
    INTEGER(KIND=I4)                                                                            ::&
    & L       , I       , J
!
    REAL   (KIND=R4)    , DIMENSION(-5:IM+6, -5:JM+6, LM)                                       ::&
    & VRBU    , VRBV    , VRBT
!
    VRBU(0:IM+1, 0:JM+1, :) = U
    VRBV(0:IM+1, 0:JM+1, :) = V
    VRBT(0:IM+1, 0:JM+1, :) = T
!
    CALL BOCOHMPI1(VRBT, LM)
    CALL BOCOVMPI1(VRBU, VRBV, LM)
!
    DO L=1,LM
        CALL SHAP_2DV(VRBU(:,:,L), 4)
!
        U(:,:,L) = U(:,:,L) + (VRBU(0:IM+1,0:JM+1,L) - U(:,:,L)) * 0.02
!
        CALL SHAP_2DV(VRBV(:,:,L), 4)
!
        V(:,:,L) = V(:,:,L) + (VRBV(0:IM+1,0:JM+1,L) - V(:,:,L)) * 0.02      
!
        CALL SHAP_2D(VRBT(:,:,L), 4)
!
        T(:,:,L) = T(:,:,L) + (VRBT(0:IM+1,0:JM+1,L) - T(:,:,L)) * 0.02
    END DO
!
    CALL BOCOVMPI(U, V, LM)
    CALL BOCOHMPI(T, LM)
!
    END SUBROUTINE SHAPIRO
!
!
!--------------------------------------------------------------------------------------------------
!  DOXYGEN
!> @brief Inserir Brief de SHAP_2DV
!> @details Inserir Details de SHAP_2DV
!> @author ORIGINATOR - ?????
!> @date ??-??-?? \n
!> @author LUCCI 
!> @date 18-03-20 \n
!> @version V1.1.0
!> @details MODERNIZATION OF THE CODE, INCLUDING:
!!                      * F77 TO F90/F95
!!                      * INDENTATION & UNIFORMIZATION CODE
!!                      * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!!                      * DOCUMENTATION WITH DOXYGEN
!!                      * OPENMP FUNCTIONALITY 
!<
!> @param[in]    NORD  - Significado de NORD
!> @param[inout] HH2D  - Significado de HH2D 
!> @details <b>Use Module:</b>
!! @arg @c F77KINDS
!! @arg @c PARMETA
!> @details <b>Driver:</b> 
!<
!> @details <b>Calls:</b>
!-------------------------------------------------------------------------------------------------- 
    SUBROUTINE SHAP_2DV(HH2D,NORD)
!--------------------------------------------------------------------------------------------------
! SUBROUTINE  SHAP_2DV
! 
! SUBPROGRAM:  SHAP_2DV - ?????
! PROGRAMMER: ????? 
! ORG: ?????
! DATE: ??-??-??
!
! ABSTRACT:  
! ?????
!
! PROGRAM HISTORY LOG:
! 87-06-??  ?????   - ORIGINATOR
! 18-03-20  LUCCI   - MODERNIZATION OF THE CODE, INCLUDING:
!                     * F77 TO F90/F95
!                     * INDENTATION & UNIFORMIZATION CODE
!                     * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!                     * DOCUMENTATION WITH DOXYGEN
!                     * OPENMP FUNCTIONALITY 
!
!
! INPUT ARGUMENT LIST:
! NORD  - 
!
! OUTPUT ARGUMENT LIST:
! NONE
!
! INPUT/OUTPUT ARGUMENT LIST:
! HH2D  - 
!
! INPUT FILES:
! NONE
!
! OUTPUT FILES:
! NONE
!
! USE MODULES: F77KINDS
!              PARMETA
!
! DRIVER     : -----
!
! CALLS      : -----
!--------------------------------------------------------------------------------------------------
    USE F77KINDS
    USE PARMETA
!
    IMPLICIT NONE
!
    INTEGER(KIND=I4)                                                      , INTENT(IN)          ::&
    & NORD
!
    REAL   (KIND=R4)    , DIMENSION(-5:IM+6, -5:JM+6)                     , INTENT(INOUT)       ::&
    & HH2D
!
    INTEGER(KIND=I4)                                                                            ::&
    & N       , I       , J
!
    DO I=1,IM-1
        CALL SHAP_1DV(HH2D(I,:), JM, NORD)
    END DO
!
    DO J=1,JM-1
        CALL SHAP_1DV(HH2D(:,J), IM, NORD)
    END DO
!
    END SUBROUTINE SHAP_2DV
!
!
!--------------------------------------------------------------------------------------------------
!  DOXYGEN
!> @brief Inserir Brief de SHAP_2DV
!> @details Inserir Details de SHAP_2DV
!> @author ORIGINATOR - ?????
!> @date ??-??-?? \n
!> @author LUCCI 
!> @date 18-03-20 \n
!> @version V1.1.0
!> @details MODERNIZATION OF THE CODE, INCLUDING:
!!                      * F77 TO F90/F95
!!                      * INDENTATION & UNIFORMIZATION CODE
!!                      * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!!                      * DOCUMENTATION WITH DOXYGEN
!!                      * OPENMP FUNCTIONALITY 
!<
!> @param[in]    IMAX  - Significado de IMAX
!> @param[in]    ORDER - Significado de ORDER
!> @param[inout] HH1D  - Significado de HH1D 
!> @details <b>Use Module:</b>
!! @arg @c F77KINDS
!! @arg @c MPPSTAFF
!> @details <b>Driver:</b> 
!<
!> @details <b>Calls:</b>
!-------------------------------------------------------------------------------------------------- 
    SUBROUTINE SHAP_1DV(HH1D, IMAX, ORDER)
!--------------------------------------------------------------------------------------------------
! SUBROUTINE  SHAP_1DV
! 
! SUBPROGRAM:  SHAP_1DV - ?????
! PROGRAMMER: ????? 
! ORG: ?????
! DATE: ??-??-??
!
! ABSTRACT:  
! ?????
!
! PROGRAM HISTORY LOG:
! 87-06-??  ?????   - ORIGINATOR
! 18-03-20  LUCCI   - MODERNIZATION OF THE CODE, INCLUDING:
!                     * F77 TO F90/F95
!                     * INDENTATION & UNIFORMIZATION CODE
!                     * REPLACEMENT OF COMMONS BLOCK FOR MODULES
!                     * DOCUMENTATION WITH DOXYGEN
!                     * OPENMP FUNCTIONALITY 
!
!
! INPUT ARGUMENT LIST:
! NORD  - 
! ORDER -
!
! OUTPUT ARGUMENT LIST:
! NONE
!
! INPUT/OUTPUT ARGUMENT LIST:
! HH1D  - 
!
! INPUT FILES:
! NONE
!
! OUTPUT FILES:
! NONE
!
! USE MODULES: F77KINDS
!              MPPSTAFF
!
! DRIVER     : -----
!
! CALLS      : -----
!--------------------------------------------------------------------------------------------------
    USE F77KINDS
    USE MPPSTAFF
!
    IMPLICIT NONE
!
    INTEGER(KIND=I4)                                                      , INTENT(IN)          ::&
    & IMAX    , ORDER
!
    REAL   (KIND=R4)    , DIMENSION(-5:IMAX+6)                            , INTENT(INOUT)       ::&
    & HH1D
!
    REAL   (KIND=R4)    , DIMENSION(-5:IMAX+6)                                                  ::&
    & HHTMP1  , HHTMP2
!
    INTEGER(KIND=I4)                                                                            ::&
    & N       , I
!
    HHTMP1 = HH1D
!
    DO N=1,ORDER
!
        DO I=-4,IMAX+5
	    HHTMP2(I) = -0.25 * (HHTMP1(I+1) - 2 * HHTMP1(I) + HHTMP1(I-1))
        END DO
!
        HHTMP1 = HHTMP2
!
    END DO
!
    HH1D = HH1D - HHTMP1
!
    END SUBROUTINE SHAP_1DV
